# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id: __init__.py 71 2008-03-10 20:57:35Z albrecht.andi $

"""Configuration"""

import gobject

from os.path import abspath, dirname, join
from configobj import ConfigObj

from gettext import gettext as _

import logging
log = logging.getLogger("CONFIG")

from cf import USER_CONF

class Config(gobject.GObject):
    """Configuration object
    
    An instance of this class is accessible through the ``config``
    attribute of an `CFApplication`_ instance.
    
    The Config class is a simplified wrapper around a ConfigObj
    instance. It merges a default configuration located as package
    data in this package with an user configuration.
    
    The ``-c`` command line switch determines which user configuration
    file is used. If it's not set, it defaults to 
    ``~/.config/crunchyfrog/config``.
    
    This wrapper provides only a getter and a setter for configuration
    values and expects that option names are dotted strings (but only by 
    convention).
    Values can be any basic Python types since it uses ConfigObj's 
    ``unrepr`` mode (Read the `ConfigObj documentation`_ for details).
    
    Plugins can connect to the `changed` signal to track configuration
    changes, i.e. the SQL editor uses this signal to reflect changes
    made through the preference dialog.
    
    .. Note:: The runtime Config instance (``app.config``) is bound
        to the application. So it is not possible to store instance
        specific data here.
    
    
    Usage example
    =============
    
        .. sourcecode:: python
        
            >>> app.config.get("foo.bar") # Not set yet, None is default
            None
            >>> app.config.set("foo.bar", True)
            >>> app.config.get("foo.bar")
            True
            >>> app.config.set("foo.bar", ["Completely", "different"]) # No type check!
            >>> print " ".join(app.config.get("foo.bar"))
            Completely different
            
    
    Signals
    =======
    
        changed
            ``def callback(config, key, value, user_oaram1, ...)``
            
            Emitted when a option has changed.
    
    
    .. _CFApplication: cf.app.CFApplication.html
    .. _ConfigObj documentation: http://www.voidspace.org.uk/python/configobj.html#unrepr-mode
    """
    
    __gsignals__ = {
        "changed" : (gobject.SIGNAL_RUN_LAST,
                     gobject.TYPE_NONE,
                     (str, gobject.TYPE_PYOBJECT)),
    }
    
    def __init__(self, app, config_file):
        """
        The constructor of this class takes two arguments:
        
        :Parameter:
            app
                `CFApplication`_ instance
            config_file
                Path to user configuration file
                
        .. _CFApplication: cf.app.CFApplication.html
        """
        self.app = app
        self.__gobject_init__() # IGNORE:E1101
        self.__conf = None
        self.__config_file = config_file
        self.__init_conf()
        self.app.register_shutdown_task(self.on_app_shutdown, 
                                        _(u"Writing configuration"))
        
    def on_app_shutdown(self, *args): # IGNORE:W0613
        """Callback: write configuration file to disk"""
        self.write()
        
    def __init_conf(self):
        """Intialize the configuration system"""
        self.__conf = ConfigObj(abspath(join(dirname(__file__), "default.cfg")),
                                unrepr=True)
        log.info("Loading configuration file %r" % self.__config_file)
        self.__conf.update(ConfigObj(self.__config_file, unrepr=True))
        
    def init(self):
        """Loads configuration"""
        pass
        
    def get(self, key, default=None):
        """Returns value or default for key"""
        return self.__conf.get(key, default)
    
    def set(self, key, value):
        """Sets key to value"""
        self.__conf[key] = value
        self.emit("changed", key, value) # IGNORE:E1101
        
    def write(self, fname=None):
        """Writes configuration file"""
        if not fname:
            fname = self.__config_file
        fp = open(fname, "w")
        self.__conf.write(fp)
        fp.close()


# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id$

"""Reference Viewer Plugin

Todo
====

 * SEGFAULT when closing an re-opening RefView 
"""

import gtk
import gobject

from cf.plugins.core import GenericPlugin
from cf.ui import GladeWidget
from cf.ui.pdock import DockItem

from gettext import gettext as _

class ReferenceViewer(GenericPlugin):
    id = "crunchyfrog.plugin.refbrowser"
    name = _(u"Reference Browser")
    description = _(u"A tiny webbrowser to view references")
    icon = "stock_help-book"
    author = "Andi Albrecht"
    license = "GPL"
    homepage = "http://cf.andialbrecht.de"
    version = "0.1"
    
    def __init__(self, app):
        GenericPlugin.__init__(self, app)
        self._viewer = dict()
        self.app.cb.connect("instance-created", self.on_instance_created)
        for instance in app.get_instances():
            self.init_instance(instance)
            
    def on_instance_created(self, app, instance):
        self.init_instance(instance)
        
    def on_toggle_viewer(self, menuitem, instance):
        if menuitem.get_active():
            gobject.idle_add(self.create_view, menuitem, instance)
        else:
            gobject.idle_add(self.remove_view, instance)
            
    def on_view_destroyed(self, view, menuitem, instance):
        menuitem.set_active(False)
        
    def create_view(self, menuitem, instance):
        view = RefView(self.app, instance)
        self._viewer[instance] = view
        item = DockItem(instance.dock, "refview", view.widget, _(u"Reference"), 
                          self.icon, gtk.POS_BOTTOM)
        instance.dock.add_item(item)
        instance.set_data("refviewer", view)
        view.widget.connect("destroy", self.on_view_destroyed, menuitem, instance)
        
    def init_instance(self, instance):
        mn_view = instance.xml.get_widget("mn_view")
        item = gtk.CheckMenuItem(_(u"Reference Viewer"))
        item.connect("activate", self.on_toggle_viewer, instance)
        item.show()
        if self.app.config.get("refviewer.visible"):
            item.set_active(True)
        mn_view.append(item)
        
    def remove_view(self, instance):
        view = self._viewer.pop(instance)
        instance.set_data("refviewer", None)
        view.destroy()
        
    def shutdown(self):
        if self._viewer.keys():
            self.app.config.set("refviewer.visible", True)
        else:
            self.app.config.set("refviewer.visible", False)
            
class RefView(GladeWidget):
    
    def __init__(self, app, instance):
        GladeWidget.__init__(self, app, "crunchyfrog", "reference_viewer")
        self.instance = instance
        
    def _setup_widget(self):
        import gtkmozembed
        self.browser = gtkmozembed.MozEmbed()
        self.xml.get_widget("refviewer_box_moz").pack_start(self.browser, True, True)
        self.list_bookmarks = self.xml.get_widget("refviewer_bookmarks")
        model = gtk.TreeStore(gobject.TYPE_PYOBJECT, str, str) # backend, url, label
        self.list_bookmarks.set_model(model)
        col = gtk.TreeViewColumn("", gtk.CellRendererText(), text=2)
        self.list_bookmarks.append_column(col)
        from cf.plugins.core import PLUGIN_TYPE_BACKEND
        for be in self.app.plugins.get_plugins(PLUGIN_TYPE_BACKEND, True):
            if be.reference and be.reference.base_url:
                iter = model.append(None)
                model.set(iter, 0, be, 1, be.reference.base_url,
                          2, be.reference.name)
                
    def on_bookmarks_row_activated(self, treeview, path, column):
        model = treeview.get_model()
        iter = model.get_iter(path)
        self.load_url(model.get_value(iter, 1))
        
    def on_load_url(self, *args):
        self.load_url(self.xml.get_widget("refviewer_entry_url").get_text())
        
    def on_toggle_favorites(self, btn):
        if btn.get_active():
            self.xml.get_widget("refviewer_sw_favorites").show_all()
        else:
            self.xml.get_widget("refviewer_sw_favorites").hide()
            
    def on_toggle_navigationbar(self, btn):
        if btn.get_active():
            self.xml.get_widget("refviewer_box_navigator").show_all()
        else:
            self.xml.get_widget("refviewer_box_navigator").hide()
            
    def load_url(self, url):
        self.xml.get_widget("refviewer_entry_url").set_text(url)
        self.browser.load_url(url)
        


from setuptools import setup
import os
import sys

if "install" in sys.argv or os.getuid() == 0:
    print "To install CrunchyFrog run"
    print 
    print "\tmake"
    print "\tmake install"
    print
    print "See README for additional information."
    sys.exit(1)

from cf import release

setup(
    name=release.appname,
    version=release.version,
    description=release.description,
    author=release.author,
    author_email=release.author_email,
    long_description=release.long_description,
    license="GPL",
    url=release.url,
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: X11 Applications :: Gnome",
        "Environment :: X11 Applications :: GTK",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Topic :: Database :: Front-Ends",
        "Topic :: Desktop Environment :: Gnome",
    ],
    include_package_data=True,
)



# -*- coding: utf-8 -*-

# crunchyfrog - a database schema browser and query tool
# Copyright (C) 2008 Andi Albrecht <albrecht.andi@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# $Id: sqlite.py 187 2008-06-10 04:39:55Z albrecht.andi $

"""SQLite3 backend"""

import gtk


from cf.backends import DBConnectError
from cf.backends.dbapi2helper import DbAPI2Connection, DbAPI2Cursor
from cf.backends.schema import *
from cf.backends import ReferenceProvider
from cf.plugins.core import DBBackendPlugin
from cf.datasources import DatasourceInfo

from gettext import gettext as _

import logging
log = logging.getLogger("SQLITE")

class SQLiteBackend(DBBackendPlugin):
    id = "crunchyfrog.backend.sqlite"
    name = _(u"SQLite3 Plugin")
    description = _(u"Provides access to SQLite3 databases")
        
    def __init__(self, app):
        DBBackendPlugin.__init__(self, app)
        log.info("Activating SQLite3 backend")
        self.schema = SQLiteSchema()
        self.reference = SQLiteReferenceProvider()

    @classmethod
    def _get_filename(cls, chooser):
        return chooser.get_filename()
        
    def shutdown(self):
        log.info("Shutting down SQLite3 backend")
    
    @classmethod
    def get_datasource_options_widgets(cls, data_widgets, initial_data=None):
        lbl = gtk.Label(_(u"Database file:"))
        lbl.set_alignment(0, 0.5)
        file_chooser = gtk.FileChooserButton(_(u"Select database file"))
        if initial_data:
            file_chooser.select_filename(initial_data.options.get("filename"))
        data_widgets["filename"] = (cls._get_filename, file_chooser)
        return data_widgets, [lbl, file_chooser]
    
    @classmethod
    def get_label(cls, datasource_info):
        return "%s (sqlite://%s)" % (datasource_info.name, datasource_info.options.get("filename"))
    
    def dbconnect(self, data):
        try:
            import sqlite3
        except ImportError:
            raise DBConnectError(_(u"Python module sqlite3 is not installed."))
        try:
            real_conn = sqlite3.connect(data["filename"])
        except StandardError, err:
            raise DBConnectError(str(err))
        return SQLite3Connection(self, self.app, real_conn)
    
    def test_connection(self, data):
        try:
            conn = self.dbconnect(data)
            conn.close()
        except DBConnectError, err:
            return str(err)
        return None
    
class SQLite3Connection(DbAPI2Connection):
    
    def get_server_info(self):
        cur = self._conn.cursor()
        cur.execute("select sqlite_version()")
        ret = cur.fetchone()[0]
        cur.close()
        return "SQLite %s" % ret
    
class SQLiteSchema(SchemaProvider):
    
    def q(self, connection, sql):
        cur = connection.cursor()._cur
        cur.execute(sql)
        return cur.fetchall()
    
    def fetch_children(self, connection, parent):
        if isinstance(parent, DatasourceInfo):
            return [TableCollection(), ViewCollection()]
        
        elif isinstance(parent, TableCollection):
            ret = []
            sql = "select name from sqlite_master where type = 'table'"
            for item in self.q(connection, sql):
                ret.append(Table(item[0]))
            return ret
        
        elif isinstance(parent, ViewCollection):
            ret = []
            sql = "select name from sqlite_master where type = 'view'"
            for item in self.q(connection, sql):
                ret.append(View(item[0]))
            return ret
        
        elif isinstance(parent, Table) or isinstance(parent, View):
            return [ColumnCollection(table=parent)]
        
        elif isinstance(parent, ColumnCollection):
            ret = []
            sql = "pragma table_info('%s')" % parent.get_data("table").name
            return [Column(item[1]) for item in self.q(connection, sql)]
        
class SQLiteReferenceProvider(ReferenceProvider):
    name = _(u"SQLite Reference")
    base_url = "http://sqlite.org/docs.html"
    
    def get_context_help_url(self, term):
        return "http://sqlite.org/lang.html"


try:
    import sqlite3
except ImportError:
    SQLiteBackend.INIT_ERROR = _(u"Python module sqlite3 required.")